from io import BytesIO
from base64 import b64encode

from matplotlib.figure import Figure

    
class ELogEntry:
    """ eLog Entry in properly formatted XML style """
    
    def __init__(self, author: str, title: str, text: str, image: Figure, severity: str):
        self.author = author
        self.title = title
        self.text = text
        self.image = image
        self.severity = severity
        
    def __str__(self) -> str:
        return self._create_entry()
    
    def _create_entry(self) -> str:
        e_log_xml_string_list = list(["<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"])
        e_log_xml_string_list.append("<entry>")
        if self.author:
            e_log_xml_string_list.append("<author>")
            e_log_xml_string_list.append(self.author)
            e_log_xml_string_list.append("</author>")
        if self.title:
            e_log_xml_string_list.append("<title>")
            e_log_xml_string_list.append(self.title)
            e_log_xml_string_list.append("</title>")
        if self.severity:
            e_log_xml_string_list.append("<severity>")
            e_log_xml_string_list.append(self.severity)
            e_log_xml_string_list.append("</severity>")
        if self.text:
            e_log_xml_string_list.append("<text>")
            e_log_xml_string_list.append(self.text.replace("<", "&lt;").replace(">", "&gt;"))
            e_log_xml_string_list.append("</text>")
        if self.image:
            e_log_xml_string_list.append("<image>")
            e_log_xml_string_list.append(self.convert_image(self.image))
            e_log_xml_string_list.append("</image>")
        e_log_xml_string_list.append("</entry>")
        return "\n".join(e_log_xml_string_list)

    @staticmethod
    def convert_image(image: Figure) -> str:
        s = BytesIO()
        image.savefig(s, format='png', bbox_inches="tight")
        return b64encode(s.getvalue()).decode("utf-8").replace("\n", "")