import subprocess
from distutils.spawn import find_executable

from .elog_entry import ELogEntry

class ELogPrinter:
    """ API to print in FLASH eLogbooks
    Precondition1: Linux commands lpr 
    Precondition2: inside DESY network for print service """
    
    ELOGS = ["exphalllog", "fl2_photlog", "fl1_camplog", "fl1_bl2log", "fl1_bl3log", "felpgmlog",
             "fl2_fl23log", "fl2_fl24log", "fl2_fl26log", "cpspectlog", "fl1_ramlog", "fl1_thzlog",
             "fl_thzsrklog", "hextoflog", "wespelog", "musixlog", "flash2_hglog", "pplaserlog",
             "fl2_pplaslog", "ttflog"]
    CUPS_SERVER = "cups-m.desy.de"
    
    def __init__(self, elog: str):
        self.elog = elog
        ELogPrinter._check_preconditions(elog)
        
    def print(self, *, author: str = None, title: str = None, 
              text: str = None, image = None, severity: str = 'INFO') -> None:   
        command = f"lpr -l -H {ELogPrinter.CUPS_SERVER} -P {self.elog}"
        entry = ELogEntry(author, title, text, image, severity)
        subprocess.run(command, 
                       shell=True,
                       check=True,
                       input=entry.__str__(),
                       encoding='iso-8859-1')
            
    @staticmethod
    def _check_preconditions(elog: str) -> None:
        ELogPrinter._check_if_linux_commands_available("lpr")
        ELogPrinter._check_if_printer_listed(elog)
     
    @staticmethod
    def _check_if_printer_listed(elog) -> None:
        if not elog in ELogPrinter.ELOGS:
            raise Exception(f"Precond.: {elog} is listed. Available are the following: {ELogPrinter.ELOGS}")
            
    @staticmethod
    def _check_if_linux_commands_available(command: str) -> None:
        """ Check hether 'command' is on PATH """
        if find_executable(command) is None:
            raise Exception(f"Precond.: {command} is available.")